@@ -18,6 +18,10 @@
 #include "log.h"
 #include "logrotate.h"
 
+#if !defined(GLOB_ABORTED) && defined(GLOB_ABEND)
+#define GLOB_ABORTED GLOB_ABEND
+#endif
+
 static char * defTabooExts[] = { ".rpmsave", ".rpmorig", "~", ",v" };
 static int defTabooCount = sizeof(defTabooExts) / sizeof(char *);
 
@@ -48,7 +52,7 @@ static int isolateValue(char * fileName, int lineNum, char * key,
     *startPtr = chptr;
 
     while (*chptr != '\n') chptr++;
-	
+
     while (isblank(*chptr)) chptr--;
 
     if (*chptr == '\n')
@@ -242,7 +246,7 @@ static int readConfigFile(char * configFile, logInfo * defConfig,
     glob_t globResult;
 
     /* FIXME: createOwner and createGroup probably shouldn't be fixed
-       length arrays -- of course, if we aren't run setuid in doesn't
+       length arrays -- of course, if we aren't run setuid it doesn't
        matter much */
 
     fd = open(configFile, O_RDONLY);
@@ -284,7 +288,7 @@ static int readConfigFile(char * configFile, logInfo * defConfig,
 
     close(fd);
 
-    /* knowing the buffer ends with a newline makes things a (bit) cleaner */
+    /* knowing the buffer ends with a newline makes things (a bit) cleaner */
     buf[length + 1] = '\0';
     buf[length] = '\n';
 
@@ -344,6 +348,18 @@ static int readConfigFile(char * configFile, logInfo * defConfig,
 	    } else if (!strcmp(start, "delaycompress")) {
 		newlog->flags |= LOG_FLAG_DELAYCOMPRESS;
 
+		*endtag = oldchar, start = endtag;
+	    } else if (!strcmp(start, "nodelaycompress")) {
+		newlog->flags &= ~LOG_FLAG_DELAYCOMPRESS;
+
+		*endtag = oldchar, start = endtag;
+	    } else if (!strcmp(start, "copytruncate")) {
+		newlog->flags |= LOG_FLAG_COPYTRUNCATE;
+
+		*endtag = oldchar, start = endtag;
+	    } else if (!strcmp(start, "nocopytruncate")) {
+		newlog->flags &= ~LOG_FLAG_COPYTRUNCATE;
+
 		*endtag = oldchar, start = endtag;
 	    } else if (!strcmp(start, "ifempty")) {
 		newlog->flags |= LOG_FLAG_IFEMPTY;
@@ -503,6 +519,14 @@ static int readConfigFile(char * configFile, logInfo * defConfig,
 		/* hmmm, we could lose memory here, but not much */
 		newlog->logAddress = NULL;
 
+		*endtag = oldchar, start = endtag;
+	    } else if (!strcmp(start, "missingok")) {
+		newlog->flags |= LOG_FLAG_MISSINGOK;
+
+		*endtag = oldchar, start = endtag;
+	    } else if (!strcmp(start, "nomissingok")) {
+		newlog->flags &= ~LOG_FLAG_MISSINGOK;
+
 		*endtag = oldchar, start = endtag;
 	    } else if (!strcmp(start, "prerotate")) {
 		*endtag = oldchar, start = endtag;
@@ -601,6 +625,19 @@ static int readConfigFile(char * configFile, logInfo * defConfig,
 		}
 
 		message(MESS_DEBUG, "olddir is now %s\n", newlog->oldDir);
+	    } else if (!strcmp(start, "extension")) {
+		*endtag = oldchar, start = endtag;
+
+		if (!isolateValue(configFile, lineNum, "extension name", &start, 
+				  &endtag)) {
+		    oldchar = *endtag, *endtag = '\0';
+
+		    newlog->extension = strdup(start);
+
+		    *endtag = oldchar, start = endtag;
+		}
+
+		message(MESS_DEBUG, "extension is now %s\n", newlog->extension);
 	    } else {
 		message(MESS_ERROR, "%s:%d unknown option '%s' "
 			    "-- ignoring line\n", configFile, lineNum, start);
@@ -618,7 +655,7 @@ static int readConfigFile(char * configFile, logInfo * defConfig,
 
 	    lineNum++;
 	    start++;
-	} else if (*start == '/') {
+	} else if (*start == '/' || *start == '"') {
 	    if (newlog != defConfig) {
 		message(MESS_ERROR, "%s:%d unexpected log filename\n", 
 			configFile, lineNum);
@@ -629,14 +666,24 @@ static int readConfigFile(char * configFile, logInfo * defConfig,
 	    *logsPtr = realloc(*logsPtr, sizeof(**logsPtr) * *numLogsPtr);
 	    newlog = *logsPtr + *numLogsPtr - 1;
 	    memcpy(newlog, defConfig, sizeof(*newlog));
-	    
-	    endtag = start;
-	    while (!isspace(*endtag)) endtag++;
+
+	    if (*start == '"') {
+		start++;
+		endtag = strchr(start, '"');
+		if (!endtag || (strchr(start, '\n') < endtag)) {
+		    message(MESS_ERROR, "%s:%d end \" expected",
+			    configFile, lineNum);
+		}
+	    } else {
+		endtag = start;
+		while (!isspace(*endtag) && *endtag != '{') endtag++;
+	    }
+
 	    oldchar = *endtag;
 	    *endtag = '\0';
 
-	    rc = glob(start, 0, globerr, &globResult);
-	    if (rc == GLOB_ABEND) return 1;
+	    rc = glob(start, GLOB_NOCHECK, globerr, &globResult);
+	    if (rc == GLOB_ABORTED) return 1;
 
 	    newlog->files = malloc(sizeof(*newlog->files) * 
 				   globResult.gl_pathc);
@@ -644,8 +691,8 @@ static int readConfigFile(char * configFile, logInfo * defConfig,
 
 	    for (i = 0; i < globResult.gl_pathc; i++) {
 		/* if we glob directories we can get false matches */
-		if (lstat(globResult.gl_pathv[i], &sb)) continue;
-		if (S_ISDIR(sb.st_mode)) continue;
+		if (!lstat(globResult.gl_pathv[i], &sb) && S_ISDIR(sb.st_mode)) 
+		    continue;
 
 		for (j = 0; j < *numLogsPtr - 1; j++) {
 		    for (k = 0; k < (*logsPtr)[j].numFiles; k++) {
@@ -670,7 +717,9 @@ static int readConfigFile(char * configFile, logInfo * defConfig,
 
 	    message(MESS_DEBUG, "reading config info for %s\n", start);
 
-	    *endtag = oldchar, start = endtag;
+	    *endtag = oldchar;
+	    if (*endtag == '"') endtag++;
+	    start = endtag;
 
 	    while (*start && isspace(*start) && *start != '{') {
 		if (*start == '\n') lineNum++;
@@ -686,8 +735,9 @@ static int readConfigFile(char * configFile, logInfo * defConfig,
 	    start++;
 	    while (isblank(*start) && *start != '\n') start++;
 	
-	    if (*start != '\n') {
-		message(MESS_ERROR, "%s:%d unexpected text after {\n");
+	    if (*start != '\n' && *start != '}') {
+		message(MESS_ERROR, "%s:%d unexpected text after {\n",
+			configFile, lineNum);
 		return 1;
 	    }
 	} else if (*start == '}') {
@@ -736,7 +786,8 @@ static int readConfigFile(char * configFile, logInfo * defConfig,
 	    }
 	} else {
 	    message(MESS_ERROR, "%s:%d lines must begin with a keyword "
-			"or a /\n", configFile, lineNum);
+			"or a filename (possibly in double quotes)\n", 
+			configFile, lineNum);
 
 	    while (*start != '\n') start++;
 	    lineNum++;
