@@ -4,6 +4,7 @@
 #include <errno.h>
 #include <fcntl.h>
 #include <getopt.h>
+#include <popt.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -62,7 +63,7 @@ static logState * findState(char * fn, logState ** statesPtr,
 static int runScript(char * logfn, char * script) {
     int fd;
     char filespec[20];
-    char cmd[50];
+    char * cmd;
     int rc;
     struct stat sb;
 
@@ -107,7 +108,8 @@ static int runScript(char * logfn, char * script) {
 
     close(fd);
 
-    sprintf(cmd, "%s %s", filespec, logfn);
+    cmd = alloca(strlen(filespec) + strlen(logfn) + 20);
+    sprintf(cmd, "/bin/sh %s %s", filespec, logfn);
     rc = system(cmd);
 
     unlink(filespec);
@@ -115,6 +117,76 @@ static int runScript(char * logfn, char * script) {
     return rc;
 }
 
+static int copyTruncate(char * currLog, char * saveLog, struct stat * sb) {
+    char buf[BUFSIZ];
+    int fdcurr = -1, fdsave = -1;
+    ssize_t cnt;
+
+    message(MESS_DEBUG, "copying %s to %s\n", currLog, saveLog);
+
+    if (!debug) {
+	if ((fdcurr = open(currLog, O_RDWR)) < 0) {
+	    message(MESS_ERROR, "error opening %s: %s\n", currLog,
+		strerror(errno));
+	    return 1;
+	}
+	if ((fdsave = open(saveLog, O_WRONLY | O_CREAT | O_TRUNC,
+		sb->st_mode)) < 0) {
+	    message(MESS_ERROR, "error creating %s: %s\n", saveLog,
+		strerror(errno));
+	    close(fdcurr);
+	    return 1;
+	}
+	if (fchmod(fdsave, sb->st_mode)) {
+	    message(MESS_ERROR, "error setting mode of %s: %s\n",
+		saveLog, strerror(errno));
+	    close(fdcurr);
+	    close(fdsave);
+	    return 1;
+	}
+	if (fchown(fdsave, sb->st_uid, sb->st_gid)) {
+	    message(MESS_ERROR, "error setting owner of %s: %s\n",
+		saveLog, strerror(errno));
+	    close(fdcurr);
+	    close(fdsave);
+	    return 1;
+	}
+	while ((cnt = read(fdcurr, buf, sizeof(buf))) > 0) {
+	    if (write(fdsave, buf, cnt) != cnt) {
+		message(MESS_ERROR, "error writing to %s: %s\n", 
+		    saveLog, strerror(errno));
+		close(fdcurr);
+		close(fdsave);
+		return 1;
+	    }
+	}
+	if (cnt != 0) {
+	    message(MESS_ERROR, "error reading %s: %s\n", 
+		currLog, strerror(errno));
+	    close(fdcurr);
+	    close(fdsave);
+	    return 1;
+	}
+    }
+
+    message(MESS_DEBUG, "truncating %s\n", currLog);
+
+    if (!debug) {
+	if (ftruncate(fdcurr, 0)) {
+	    message(MESS_ERROR, "error truncating %s: %s\n", currLog,
+		strerror(errno));
+	    close(fdcurr);
+	    close(fdsave);
+	    return 1;
+	} else {
+	    close(fdcurr);
+	    close(fdsave);
+	}
+    }
+
+    return 0;
+}
+
 int rotateSingleLog(logInfo * log, int logNum, logState ** statesPtr, 
 		    int * numStatesPtr, FILE * errorFile) {
     struct stat sb;
@@ -124,7 +196,8 @@ int rotateSingleLog(logInfo * log, int logNum, logState ** statesPtr,
     char * disposeName;
     char * finalName;
     char * tmp;
-    char * ext = "";
+    char * compext = "";
+    char * fileext = "";
     int hasErrors = 0;
     int doRotate = 0;
     int i;
@@ -138,9 +211,14 @@ int rotateSingleLog(logInfo * log, int logNum, logState ** statesPtr,
 
     message(MESS_DEBUG, "rotating file %s\n", log->files[logNum]);
 
-    if (log->flags & LOG_FLAG_COMPRESS) ext = COMPRESS_EXT;
+    if (log->flags & LOG_FLAG_COMPRESS) compext = COMPRESS_EXT;
     
     if (stat(log->files[logNum], &sb)) {
+	if ((log->flags & LOG_FLAG_MISSINGOK) && (errno == ENOENT)) {
+	    message(MESS_DEBUG, "file %s does not exist -- skipping\n", 
+		    log->files[logNum]);
+	    return 0;
+	}
 	fprintf(errorFile, "stat of %s failed: %s\n", log->files[logNum],
 		strerror(errno));
 	hasErrors = 1;
@@ -151,6 +229,10 @@ int rotateSingleLog(logInfo * log, int logNum, logState ** statesPtr,
 
 	if (log->criterium == ROT_SIZE) {
 	    doRotate = (sb.st_size >= log->threshhold);
+	}
+	else if (log->criterium == ROT_FORCE) {
+	    /* user forced rotation of logs from command line */
+	    doRotate = 1;
 	} else if (state->lastRotated.tm_year != now.tm_year || 
 		   state->lastRotated.tm_mon != now.tm_mon ||
 		   state->lastRotated.tm_mday != now.tm_mday) {
@@ -203,7 +285,7 @@ int rotateSingleLog(logInfo * log, int logNum, logState ** statesPtr,
 		dirName = strdup(log->oldDir);
 	    else
 		dirName = ourDirName(log->files[logNum]);
-	    baseName = ourBaseName(log->files[logNum]);
+	    baseName = strdup(ourBaseName(log->files[logNum]));
 
 	    oldName = alloca(strlen(dirName) + strlen(baseName) + 
 				strlen(log->files[logNum]) + 10);
@@ -212,12 +294,24 @@ int rotateSingleLog(logInfo * log, int logNum, logState ** statesPtr,
 	    disposeName = alloca(strlen(dirName) + strlen(baseName) + 
 				strlen(log->files[logNum]) + 10);
 
+	    if (log->extension &&
+		    strncmp(&baseName[strlen(baseName)-strlen(log->extension)],
+		    log->extension, strlen(log->extension)) == 0) {
+		char *tempstr;
+		fileext = log->extension;
+		tempstr = calloc(strlen(baseName)-strlen(log->extension), sizeof(char));
+		strncpy(tempstr, baseName,
+			strlen(baseName)-strlen(log->extension));
+		free(baseName);
+		baseName = tempstr;
+	    }
+
 	    /* First compress the previous log when necessary */
 	    if (log->flags & LOG_FLAG_COMPRESS &&
 			log->flags & LOG_FLAG_DELAYCOMPRESS) {
 		struct stat sbprev;
 
-		sprintf(oldName, "%s/%s.1", dirName, baseName);
+		sprintf(oldName, "%s/%s.1%s", dirName, baseName, fileext);
 		if (stat(oldName, &sbprev)) {
 		    message(MESS_DEBUG, "previous log %s does not exist\n",
 					oldName);
@@ -237,8 +331,8 @@ int rotateSingleLog(logInfo * log, int logNum, logState ** statesPtr,
 		}
 	    }
 
-	    sprintf(oldName, "%s/%s.%d%s", dirName, baseName,
-		    log->rotateCount + 1, ext);
+	    sprintf(oldName, "%s/%s.%d%s%s", dirName, baseName,
+		    log->rotateCount + 1, fileext, compext);
 
 	    strcpy(disposeName, oldName);
 
@@ -246,7 +340,8 @@ int rotateSingleLog(logInfo * log, int logNum, logState ** statesPtr,
 		tmp = newName;
 		newName = oldName;
 		oldName = tmp;
-		sprintf(oldName, "%s/%s.%d%s", dirName, baseName, i, ext);
+		sprintf(oldName, "%s/%s.%d%s%s", dirName, baseName, i,
+			fileext, compext);
 
 		message(MESS_DEBUG, "renaming %s to %s\n", oldName, newName);
 
@@ -265,7 +360,7 @@ int rotateSingleLog(logInfo * log, int logNum, logState ** statesPtr,
 	    finalName = oldName;
 
 	    /* note: the gzip extension is *not* used here! */
-	    sprintf(finalName, "%s/%s.1", dirName, baseName);
+	    sprintf(finalName, "%s/%s.1%s", dirName, baseName, fileext);
 
 	    /* if the last rotation doesn't exist, that's okay */
 	    if (!debug && access(disposeName, F_OK)) {
@@ -275,6 +370,7 @@ int rotateSingleLog(logInfo * log, int logNum, logState ** statesPtr,
 	    } 
 
 	    free(dirName);
+	    free(baseName);
 	}
 
 	if (!hasErrors && log->logAddress && disposeName) {
@@ -284,11 +380,12 @@ int rotateSingleLog(logInfo * log, int logNum, logState ** statesPtr,
 			     strlen(UNCOMPRESS_PIPE));
 
 	    if (log->flags & LOG_FLAG_COMPRESS)
-		sprintf(command, "%s < %s | /bin/mail -s '%s' %s", 
-			    UNCOMPRESS_PIPE, disposeName, log->files[logNum],
+		sprintf(command, "%s < %s | %s '%s' %s", 
+			    UNCOMPRESS_PIPE, disposeName, MAIL_COMMAND,
+			    log->files[logNum],
 			    log->logAddress);
 	    else
-		sprintf(command, "/bin/mail -s '%s' %s < %s", disposeName, 
+		sprintf(command, "%s '%s' %s < %s", MAIL_COMMAND, disposeName, 
 			    log->logAddress, disposeName);
 
 	    message(MESS_DEBUG, "executing: \"%s\"\n", command);
@@ -322,15 +419,19 @@ int rotateSingleLog(logInfo * log, int logNum, logState ** statesPtr,
 		}
 	    }
 
-	    message(MESS_DEBUG, "renaming %s to %s\n", log->files[logNum], 
+	    if (!(log->flags & LOG_FLAG_COPYTRUNCATE)) {
+		message(MESS_DEBUG, "renaming %s to %s\n", log->files[logNum], 
 		    finalName);
 
-	    if (!debug && !hasErrors && rename(log->files[logNum], finalName)) {
-		fprintf(errorFile, "failed to rename %s to %s: %s\n",
+		if (!debug && !hasErrors &&
+			rename(log->files[logNum], finalName)) {
+		    fprintf(errorFile, "failed to rename %s to %s: %s\n",
 			log->files[logNum], finalName, strerror(errno));
+		}
 	    }
 
-	    if (!hasErrors && log->flags & LOG_FLAG_CREATE) {
+	    if (!hasErrors && log->flags & LOG_FLAG_CREATE &&
+			!(log->flags & LOG_FLAG_COPYTRUNCATE)) {
 		if (log->createUid == NO_UID)
 		    createUid = sb.st_uid;
 		else
@@ -356,6 +457,12 @@ int rotateSingleLog(logInfo * log, int logNum, logState ** statesPtr,
 				log->files[logNum], strerror(errno));
 			hasErrors = 1;
 		    } else {
+			if (fchmod(fd, createMode)) {
+			    message(MESS_ERROR, "error setting mode of "
+				    "%s: %s\n", log->files[logNum], 
+				     strerror(errno));
+			    hasErrors = 1;
+			}
 			if (fchown(fd, createUid, createGid)) {
 			    message(MESS_ERROR, "error setting owner of "
 				    "%s: %s\n", log->files[logNum], 
@@ -368,6 +475,11 @@ int rotateSingleLog(logInfo * log, int logNum, logState ** statesPtr,
 		}
 	    }
 
+	    if (!hasErrors && log->flags & LOG_FLAG_COPYTRUNCATE) {
+		hasErrors = copyTruncate(log->files[logNum], finalName, &sb);
+
+	    }
+
 	    if (!hasErrors && log->post) {
 		message(MESS_DEBUG, "running postrotate script\n");
 		if (runScript(log->files[logNum], log->post)) {
@@ -399,13 +511,17 @@ int rotateSingleLog(logInfo * log, int logNum, logState ** statesPtr,
     return hasErrors;
 }
 
-int rotateLogSet(logInfo * log, logState ** statesPtr, int * numStatesPtr) {
+int rotateLogSet(logInfo * log, logState ** statesPtr, int * numStatesPtr, 
+		 int force) {
     FILE * errorFile;
     char * errorFileName = NULL;
     int oldstderr = 0, newerr;
     int i;
     int hasErrors = 0;
 
+    if (force)
+	log->criterium = ROT_FORCE;
+
     message(MESS_DEBUG, "rotating pattern: %s ", log->pattern);
     switch (log->criterium) {
       case ROT_DAYS:
@@ -420,6 +536,9 @@ int rotateLogSet(logInfo * log, logState ** statesPtr, int * numStatesPtr) {
       case ROT_SIZE:
 	message(MESS_DEBUG, "%d bytes ", log->threshhold);
 	break;
+      case ROT_FORCE:
+	message(MESS_DEBUG, "forced from command line ");
+	break;
     }
 
     if (log->rotateCount) 
@@ -488,8 +607,8 @@ int rotateLogSet(logInfo * log, logState ** statesPtr, int * numStatesPtr) {
 
 	    command = alloca(strlen(errorFileName) + strlen(log->errAddress) +
 				50);
-	    sprintf(command, "/bin/mail -s 'errors rotating logs' %s < %s\n",
-		    log->errAddress, errorFileName);
+	    sprintf(command, "%s 'errors rotating logs' %s < %s\n",
+		    MAIL_COMMAND, log->errAddress, errorFileName);
 	    if (system(command)) {
 		message(MESS_ERROR, "error mailing error log to %s -- errors "
 			"left in %s\n", log->errAddress, errorFileName);
@@ -587,13 +706,13 @@ static int readState(char * stateFilename, logState ** statesPtr,
 
 	if (sscanf(buf, "%s %d-%d-%d\n", buf2, &year, &month, &day) != 4) {
 	    message(MESS_ERROR, "bad line %d in state file %s\n", 
-		    stateFilename);
+		    line, stateFilename);
 	    fclose(f);
 	    return 1;
 	}
 
 	/* Hack to hide earlier bug */
-	if (year != 1900 && (year < 1996 || year > 2100)) {
+	if ((year != 1900) && (year < 1996 || year > 2100)) {
 	    message(MESS_ERROR, "bad year %d for file %s in state file %s\n",
 			year, buf2, stateFilename);
 	    fclose(f);
@@ -634,39 +753,41 @@ static int readState(char * stateFilename, logState ** statesPtr,
 
 }
 
-void usage(void) {
-    fprintf(stderr, "logrotate " VERSION 
-		" - Copyright (C) 1995 - Red Hat Software\n");
-    fprintf(stderr, "This may be freely redistributed under the terms of "
-		"the GNU Public License\n\n");
-    fprintf(stderr, "usage: logrotate [-dv] [-s|--state <file>] <config_file>+\n");
-    exit(1);
-}
-
 int main(int argc, char ** argv) {
-    logInfo defConfig = { NULL, NULL, 0, NULL, ROT_SIZE, 1024 * 1024, 0, 0, 
-			  NULL, NULL, NULL, LOG_FLAG_IFEMPTY,
-			  NO_MODE, NO_UID, NO_GID };
+    logInfo defConfig = { NULL, NULL, 0, NULL, ROT_SIZE, 
+			  /* threshHold */ 1024 * 1024, 0,
+			  /* pre */ NULL, NULL, NULL, NULL, 
+			  /* extension */ NULL, 
+			  /* flags */ LOG_FLAG_IFEMPTY,
+			  /* createMode */ NO_MODE, NO_UID, NO_GID };
     int numLogs = 0, numStates = 0;
+    int force = 0;
     logInfo * logs = NULL;
     logState * states = NULL;
     char * stateFile = STATEFILE;
     int i;
     int rc = 0;
-    int arg, long_index;
-    struct option options[] = {
-	{ "debug", 0, 0, 'd' },
-	{ "state", 1, 0, 's' },
-	{ "verbose", 0, 0, 'v' },
-	{ 0, 0, 0, 0 } 
+    int arg;
+    char ** files, ** file;
+    poptContext optCon;
+    struct poptOption options[] = {
+	{ "force", 'f', 0 , &force, 0, "Force file rotation" },
+	{ "debug", 'd', 0, 0, 'd', 
+		"Don't do anything, just test (implies -v)" },
+	{ "state", 's', POPT_ARG_STRING, &stateFile, 0, "Path of state file",
+		"statefile" },
+	{ "verbose", 'v', 0, 0, 'v', "Display messages during rotation" },
+	POPT_AUTOHELP
+	{ 0, 0, 0, 0, 0 } 
     };
 
     logSetLevel(MESS_NORMAL);
 
-    while (1) {
-        arg = getopt_long(argc, argv, "ds:v", options, &long_index);
-        if (arg == -1) break;
+    optCon = poptGetContext("logrotate", argc, argv, options,0);
+    poptReadDefaultConfig(optCon, 1);
+    poptSetOtherOptionHelp(optCon, "[OPTION...] <configfile>");
 
+    while ((arg = poptGetNextOpt(optCon)) >= 0) {
         switch (arg) {
 	  case 'd':
 	    debug = 1;
@@ -674,29 +795,30 @@ int main(int argc, char ** argv) {
 	  case 'v':
 	    logSetLevel(MESS_DEBUG);
 	    break;
-
-	  case 's':
-	    stateFile = optarg;
-	    break;
-
-          case '?':
-	    usage();
-            break;
 	}
     }
 
-    if (optind == argc) {
-	usage();
+    if (arg < -1) {
+	fprintf(stderr, "logrotate: bad argument %s: %s\n", 
+		poptBadOption(optCon, POPT_BADOPTION_NOALIAS), 
+		poptStrerror(rc));
+	return 2;
     }
 
-    /* reset the umask so new files get created properly */
-    umask(0);
+    files = poptGetArgs(optCon);
+    if (!files) {
+	fprintf(stderr, "logrotate " VERSION 
+		    " - Copyright (C) 1995 - Red Hat Software\n");
+	fprintf(stderr, "This may be freely redistributed under the terms of "
+		    "the GNU Public License\n\n");
+	poptPrintUsage(optCon, stderr, 0);
+	exit(1);
+    }
 
-    while (optind < argc) {
-	if (readConfigPath(argv[optind], &defConfig, &logs, &numLogs)) {
+    for (file = files; *file; file++) {
+	if (readConfigPath(*file, &defConfig, &logs, &numLogs)) {
 	    exit(1);
 	}
-	optind++;
     }
 
     if (readState(stateFile, &states, &numStates)) {
@@ -706,7 +828,7 @@ int main(int argc, char ** argv) {
     message(MESS_DEBUG, "Handling %d logs\n", numLogs);
 
     for (i = 0; i < numLogs; i++) {
-	rc |= rotateLogSet(logs + i, &states, &numStates);
+	rc |= rotateLogSet(logs + i, &states, &numStates, force);
     }
 
     if (!debug && writeState(stateFile, states, numStates)) {
